// OutlookColleagues
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the core functionality for the OutlookColleagues plug-in application.

#region Using declarations

using System;
using System.Collections;
using System.IO;
using Microsoft.Office.Core;
using Extensibility;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.Office.Interop.Outlook;

#endregion

namespace OutlookColleaguesNS {

	public class ContactItem {
		
		// This class represents an email recipient - really just a data structure.

		#region Variables and initialization code

		public string strName, strLabel, strAddress;
		public int intCount;
		public DateTime dtLastUsed;
		public ContactItem(int intCount, DateTime dtLastUsed, string strName, string strLabel, string strAddress) {
			this.intCount = intCount;
			this.dtLastUsed = dtLastUsed;
			this.strName = strName;
			this.strLabel = strLabel;
			this.strAddress = strAddress;
		}

		#endregion

	}

	[GuidAttribute("60ADB309-3192-4CAB-ACAD-2161DF7C3022"), ProgId("OutlookColleaguesNS.OutlookColleagues")]
	public class OutlookColleagues : Object, Extensibility.IDTExtensibility2 {

		// This class is the core component of OutlookColleagues.

		#region Variables and initialization code

		internal CommandBar cmdBar;
		internal CommandBarButton[] cmdBarButtons;
		internal int intShowButtons;
		internal ArrayList contactItems;
		internal Microsoft.Office.Interop.Outlook.Application applicationObject;
		internal int intMaxContacts;
		internal object addInInstance;
		internal MAPIFolder folderDrafts = null;

		public OutlookColleagues() {
			intMaxContacts = 500;
			intShowButtons = 8;
		}

		#endregion

		#region COM registration and deregistration functions

		public void OnConnection(object application, Extensibility.ext_ConnectMode connectMode, object addInInst, ref System.Array custom) {
			string strRead;
			StreamReader stream = null;
			// create application objects
			applicationObject = (Microsoft.Office.Interop.Outlook.Application) application;
			addInInstance = addInInst;
			if (connectMode != ext_ConnectMode.ext_cm_Startup)
				OnStartupComplete(ref custom);
			// get and retain pointer to Drafts folder
			Microsoft.Office.Interop.Outlook._NameSpace ns = applicationObject.GetNamespace("MAPI");
			folderDrafts = ns.GetDefaultFolder(Microsoft.Office.Interop.Outlook.OlDefaultFolders.olFolderDrafts);
			// read entries from list file
			contactItems = new ArrayList();
			if (System.IO.File.Exists(System.Reflection.Assembly.GetExecutingAssembly().Location.Substring(0, System.Reflection.Assembly.GetExecutingAssembly().Location.Length - 21) + "List.txt")) {
				try {
					stream = new StreamReader(System.Reflection.Assembly.GetExecutingAssembly().Location.Substring(0, System.Reflection.Assembly.GetExecutingAssembly().Location.Length - 21) + "List.txt");
					for (int i = 0; i < 500; i++) {
						strRead = stream.ReadLine();
						if (strRead != null) {  // got at least some chars
							string[] parse = strRead.Split(new char[] { '|' }, 5);  // split string into no more than four fields based on | delimiter
							if (parse.Length == 5)  // proper string read
								contactItems.Add(new ContactItem(int.Parse(parse[0]), DateTime.Parse(parse[1]), parse[2], parse[3], parse[4]));
						}
					}
				}
				catch (System.Exception ex) {
					if (!(ex is System.IO.FileNotFoundException))
						MessageBox.Show("OutlookColleagues encountered the following error:\r\n" + ex.Message, "OutlookColleagues startup error");
				}
				finally {
					if (stream != null)
						stream.Close();
				}
			}
		}

		public void OnStartupComplete(ref System.Array custom)	{
			bool found = false;
			CommandBars cmdBars = null;
			try {
				cmdBarButtons = new CommandBarButton[21];
				object missing = System.Reflection.Missing.Value;
				object activeExplorer = applicationObject.GetType().InvokeMember("ActiveExplorer",System.Reflection.BindingFlags.GetProperty,null,applicationObject,null);
				applicationObject.ItemSend += new ApplicationEvents_11_ItemSendEventHandler(DoItemSend);
				cmdBars = (CommandBars) activeExplorer.GetType().InvokeMember("CommandBars",System.Reflection.BindingFlags.GetProperty,null,activeExplorer,null);
				foreach (CommandBar c in cmdBars) {
					if (c.Name == "OutlookColleagues") {  // found bar - try to get buttons.
						for (int i = 1; i <= 21; i++)
							cmdBarButtons[i - 1] = (CommandBarButton) c.Controls[i];
						found = true;
						break;
					}
				}
			}
			catch (System.Exception ex) {
				found = false;
			}
			if ((found == false) && (cmdBars != null)) {  // bar does not yet exist - add it and the buttons.
				try {
					cmdBar = cmdBars.Add("OutlookColleagues", MsoBarPosition.msoBarTop, false, false);
					cmdBar.Visible = true;
					for (int i = 0; i < 21; i++) {
						cmdBarButtons[i] = (CommandBarButton) cmdBar.Controls.Add(1, System.Reflection.Missing.Value, System.Reflection.Missing.Value,
							System.Reflection.Missing.Value, System.Reflection.Missing.Value);
						cmdBarButtons[i].Style = MsoButtonStyle.msoButtonCaption;
						found = true;
					}
				} catch (System.Exception ex) {
					MessageBox.Show(ex.Message);
				}
			}
			if (found == true) {  // add event handlers
				for (int i = 0; i < 21; i++)
					cmdBarButtons[i].Click += new _CommandBarButtonEvents_ClickEventHandler(Email_Click);
				UpdateButtons();  // add button titles
			}
		}

		public void OnBeginShutdown(ref System.Array custom) {
			StreamWriter stream = null;
			try {
				stream = new StreamWriter(System.Reflection.Assembly.GetExecutingAssembly().Location.Substring(0, System.Reflection.Assembly.GetExecutingAssembly().Location.Length - 21) + @"List.txt");
				for (int i = 0; i < contactItems.Count; i++) {
					ContactItem item = (ContactItem) contactItems[i];
					stream.WriteLine(item.intCount.ToString() + "|" + item.dtLastUsed.ToString() + "|" + item.strName + "|" + item.strLabel + "|" + item.strAddress);
				}
			}
			catch (System.Exception ex) {
				MessageBox.Show("OutlookColleagues encountered the following error:\r\n" + ex.Message, "OutlookColleagues shutdown error");
			}
			finally {
				if (stream != null)
 				stream.Close();
			}
		}

		public void OnDisconnection(Extensibility.ext_DisconnectMode disconnectMode, ref System.Array custom)	{
			if (disconnectMode != ext_DisconnectMode.ext_dm_HostShutdown)
				OnBeginShutdown(ref custom);
			applicationObject = null;
		}

		public void OnAddInsUpdate(ref System.Array array) {
			// an empty stub function - must be present for properly subclassing the 
		}

		#endregion

		#region Button-click event handlers

		private void Email_Click(CommandBarButton button, ref bool throwaway) {
			if (button.Index == 21) {
				(new OutlookColleaguesConfiguration(this)).ShowDialog();
				UpdateButtons();
			}
			else {
				Microsoft.Office.Interop.Outlook._MailItem message;
				string[] strRecipients = ((ContactItem) contactItems[button.Index - 1]).strAddress.Split(new char[] { ';' } );
				if ((strRecipients.Length > 0) && (button.Index - 1 < contactItems.Count) && (folderDrafts != null)) {  // we have at least one address and can proceed.
					if (folderDrafts.Items.Count == 0) {  // create new draft
						message = (Microsoft.Office.Interop.Outlook._MailItem) applicationObject.CreateItem(Microsoft.Office.Interop.Outlook.OlItemType.olMailItem);
						message.Move(folderDrafts);
					}
					else  // pull first draft and add to list
						message = (Microsoft.Office.Interop.Outlook._MailItem) folderDrafts.Items[1];
					foreach (string strRecipient in strRecipients)
							message.Recipients.Add(strRecipient);
					message.Display(false);
				}
			}
		}

		#endregion

		#region Helper functions

		internal void DoItemSend(object item, ref bool cancel) {
			// This function monitors sent emails and extracts information about recipients.
			int count;
			if (item is MailItem) {
				bool updateButtons = false;  // don't need to update top buttons
				foreach (Recipient r in ((MailItem) item).Recipients) {
					count = 0;
					ContactItem updatedItem = null;
					foreach (object itemobject in contactItems) {
						ContactItem contact = (ContactItem) itemobject;
						if (contact.strAddress == r.Address) {
							contact.intCount++;
							contact.dtLastUsed = System.DateTime.Now;
							updatedItem = contact;
							if ((count < 20) || (updatedItem.intCount > ((ContactItem) contactItems[9]).intCount)) {  // re-sort into list
								for (int i = 0; i < count; i++) {
									if (updatedItem.intCount > ((ContactItem) contactItems[i]).intCount) {  // reorganize list
										contactItems.RemoveAt(count);  // remove contact
										contactItems.Insert(i, updatedItem);  // re-add to list
										updateButtons = true;  // remember to re-label buttons at the end
										break;
									}
								}
							}
							break;
						}
						count++;
					}
					if (updatedItem == null) {  // no item found - create new item
						while (contactItems.Count >= this.intMaxContacts) {  // remove oldest contact
							ContactItem oldest = (ContactItem) contactItems[0];
							foreach (object contactItem in contactItems) {
								if (oldest.dtLastUsed > ((ContactItem) contactItem).dtLastUsed)
									oldest = (ContactItem) contactItem;  // pick this contact as the oldest
							}
							contactItems.Remove(oldest);
						}
						string label;
						if (r.Name.IndexOf("@") > -1)
 						label = (r.Name.Substring(0, Math.Min(r.Name.IndexOf("@"), Math.Min(r.Name.Length, 6))));
						else
							label = (r.Name.Substring(0, Math.Min(r.Name.Length, 6)));
						contactItems.Add(new ContactItem(1, System.DateTime.Now, r.Name, label, r.Address));
						if (contactItems.Count <= 20)
							updateButtons = true;
					}
				}
				if (updateButtons)  // if any of the top 20 changed, update the button count.
					UpdateButtons();
			}
		}

		internal void UpdateButtons() {
			// This function updates all of the buttons in the OutlookColleagues plug-in.
			int i = 0;
			for (; i < Math.Min(20, contactItems.Count); i++) {
				cmdBarButtons[i].Caption = ((ContactItem) contactItems[i]).strLabel + " |";
				cmdBarButtons[i].Visible = (i < intShowButtons);
			}
			for (; i < 20; i++) {
				cmdBarButtons[i].Caption = "(empty) |";
				cmdBarButtons[i].Visible = (i < intShowButtons);
			}
			cmdBarButtons[20].Caption = "Options";
		}

		#endregion

	}
}
